"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cell_id_helper_1 = __importDefault(require("./cell-id-helper"));
const general_helper_1 = require("./general-helper");
const xml_helper_1 = require("./xml-helper");
class ModifyXmlHelper {
    constructor(root) {
        this.root = root;
    }
    modify(tags, root) {
        root = root || this.root;
        for (const tag in tags) {
            const modifier = tags[tag];
            if (general_helper_1.GeneralHelper.propertyExists(modifier, 'collection')) {
                const modifies = general_helper_1.GeneralHelper.arrayify(modifier.collection);
                const collection = root.getElementsByTagName(tag);
                Object.values(modifies).forEach((modifyXml) => modifyXml(collection));
                return;
            }
            const index = modifier.index || 0;
            this.assertNode(root.getElementsByTagName(tag), index, tag, modifier);
            const element = root.getElementsByTagName(tag)[index];
            if (general_helper_1.GeneralHelper.propertyExists(modifier, 'modify')) {
                const modifies = general_helper_1.GeneralHelper.arrayify(modifier.modify);
                Object.values(modifies).forEach((modifyXml) => modifyXml(element));
            }
            if (general_helper_1.GeneralHelper.propertyExists(modifier, 'children')) {
                this.modify(modifier.children, element);
            }
        }
    }
    assertNode(collection, index, tag, info) {
        if (!collection[index]) {
            if (collection[collection.length - 1] === undefined) {
                console.log(info);
                throw new Error(`Index ${index} not found at "${tag}"`);
            }
            const tplNode = collection[collection.length - 1];
            const newChild = tplNode.cloneNode(true);
            xml_helper_1.XmlHelper.insertAfter(newChild, tplNode);
        }
    }
}
exports.default = ModifyXmlHelper;
ModifyXmlHelper.text = (label) => (element) => {
    element.firstChild.textContent = String(label);
};
ModifyXmlHelper.value = (value, index) => (element) => {
    element.getElementsByTagName('c:v')[0].firstChild.textContent = String(value);
    if (index !== undefined) {
        element.setAttribute('idx', String(index));
    }
};
ModifyXmlHelper.attribute = (attribute, value) => (element) => {
    element.setAttribute(attribute, String(value));
};
ModifyXmlHelper.range = (series, length) => (element) => {
    const range = element.firstChild.textContent;
    element.firstChild.textContent = cell_id_helper_1.default.setRange(range, series, length);
};
//# sourceMappingURL=modify-xml-helper.js.map