"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModifyChart = void 0;
const xml_helper_1 = require("../helper/xml-helper");
const cell_id_helper_1 = __importDefault(require("../helper/cell-id-helper"));
const modify_xml_helper_1 = __importDefault(require("../helper/modify-xml-helper"));
class ModifyChart {
    constructor(chart, workbook, data, slot) {
        this.series = (index, children) => {
            return {
                'c:ser': {
                    index: index,
                    children: children,
                },
            };
        };
        this.seriesId = (series) => {
            return {
                'c:idx': {
                    modify: modify_xml_helper_1.default.attribute('val', series),
                },
                'c:order': {
                    modify: modify_xml_helper_1.default.attribute('val', series + 1),
                },
            };
        };
        this.seriesLabel = (label, series) => {
            return {
                'c:f': {
                    modify: modify_xml_helper_1.default.range(series + 1),
                },
                'c:v': {
                    modify: modify_xml_helper_1.default.text(label),
                },
            };
        };
        this.point = (r, c, value) => {
            return {
                children: {
                    'c:pt': {
                        index: r,
                        modify: modify_xml_helper_1.default.value(value, r),
                    },
                    'c:f': {
                        modify: modify_xml_helper_1.default.range(c, this.height),
                    },
                    'c:ptCount': {
                        modify: modify_xml_helper_1.default.attribute('val', this.height),
                    },
                },
            };
        };
        this.data = data;
        this.chart = new modify_xml_helper_1.default(chart);
        this.workbook = new modify_xml_helper_1.default(workbook.sheet);
        this.workbookTable = new modify_xml_helper_1.default(workbook.table);
        this.sharedStrings = workbook.sharedStrings;
        this.columns = this.setColumns(slot);
        this.height = this.data.categories.length;
        this.width = this.columns.length;
    }
    modify() {
        this.setValues();
        this.setSeries();
        this.sliceChartSpace();
        this.setWorkbook();
        this.sliceWorkbook();
        this.setWorkbookTable();
        this.sliceWorkbookTable();
    }
    setColumns(slot) {
        const columns = [];
        slot.forEach((slot) => {
            const series = slot.series;
            const index = slot.index;
            const targetCol = slot.targetCol;
            const label = slot.label ? slot.label : series.label;
            const mapData = slot.mapData !== undefined ? slot.mapData : (point) => point;
            const isStrRef = slot.isStrRef !== undefined ? slot.isStrRef : true;
            const worksheetCb = (point, r, category) => {
                return this.workbook.modify(this.rowValues(r, targetCol, mapData(point, category)));
            };
            const chartCb = slot.type !== undefined &&
                this[slot.type] !== undefined &&
                typeof this[slot.type] === 'function'
                ? (point, r, category) => {
                    return this[slot.type](r, targetCol, point, category, slot.tag, mapData);
                }
                : null;
            const column = {
                series: index,
                label: label,
                worksheet: worksheetCb,
                chart: chartCb,
                isStrRef: isStrRef,
            };
            columns.push(column);
        });
        return columns;
    }
    setValues() {
        this.data.categories.forEach((category, c) => {
            this.columns
                .filter((col) => col.chart)
                .forEach((col) => {
                if (category.values[col.series] === undefined) {
                    throw new Error(`No value for category "${category.label}" at series "${col.label}".`);
                }
                col.modTags = col.chart(category.values[col.series], c, category);
                this.chart.modify(this.series(col.series, col.modTags));
            });
        });
    }
    setSeries() {
        this.columns.forEach((column, colId) => {
            if (column.isStrRef === true) {
                this.chart.modify(this.series(column.series, Object.assign(Object.assign({}, this.seriesId(column.series)), this.seriesLabel(column.label, colId))));
            }
        });
    }
    sliceChartSpace() {
        this.chart.modify({
            'c:plotArea': this.slice('c:ser', this.data.series.length),
        });
        this.columns
            .filter((column) => column.modTags)
            .forEach((column) => {
            const sliceMod = {};
            Object.keys(column.modTags).forEach((tag) => {
                sliceMod[tag] = this.slice('c:pt', this.height);
            });
            this.chart.modify(this.series(column.series, sliceMod));
        });
    }
    setWorkbook() {
        this.workbook.modify(this.spanString());
        this.workbook.modify(this.rowAttributes(0, 1));
        this.data.categories.forEach((category, c) => {
            const r = c + 1;
            this.workbook.modify(this.rowLabels(r, category.label));
            this.workbook.modify(this.rowAttributes(r, r + 1));
            this.columns.forEach((addCol) => addCol.worksheet(category.values[addCol.series], r, category));
        });
        this.columns.forEach((addCol, s) => {
            this.workbook.modify(this.colLabel(s + 1, addCol.label));
        });
    }
    sliceWorkbook() {
        this.data.categories.forEach((category, c) => {
            const r = c + 1;
            this.workbook.modify({
                row: Object.assign({ index: r }, this.slice('c', this.width + 1)),
            });
        });
        this.workbook.modify({
            row: Object.assign({}, this.slice('c', this.width + 1)),
        });
        this.workbook.modify({
            sheetData: this.slice('row', this.height + 1),
        });
    }
    defaultSeries(r, targetCol, point, category) {
        return {
            'c:val': this.point(r, targetCol, point),
            'c:cat': this.point(r, 0, category.label),
        };
    }
    xySeries(r, targetCol, point, category) {
        return {
            'c:xVal': this.point(r, targetCol, point),
            'c:yVal': this.point(r, 1, category.y),
        };
    }
    customSeries(r, targetCol, point, category, tag, mapData) {
        return {
            [tag]: this.point(r, targetCol, mapData(point, category)),
        };
    }
    colLabel(c, label) {
        return {
            row: {
                modify: modify_xml_helper_1.default.attribute('spans', `1:${this.width}`),
                children: {
                    c: {
                        index: c,
                        modify: modify_xml_helper_1.default.attribute('r', cell_id_helper_1.default.getCellAddressString(c, 0)),
                        children: this.sharedString(label),
                    },
                },
            },
        };
    }
    rowAttributes(r, rowId) {
        return {
            row: {
                index: r,
                modify: [
                    modify_xml_helper_1.default.attribute('spans', `1:${this.width}`),
                    modify_xml_helper_1.default.attribute('r', String(rowId)),
                ],
            },
        };
    }
    rowLabels(r, label) {
        return {
            row: {
                index: r,
                children: {
                    c: {
                        modify: modify_xml_helper_1.default.attribute('r', cell_id_helper_1.default.getCellAddressString(0, r)),
                        children: this.sharedString(label),
                    },
                },
            },
        };
    }
    rowValues(r, c, value) {
        return {
            row: {
                index: r,
                children: {
                    c: {
                        index: c,
                        modify: modify_xml_helper_1.default.attribute('r', cell_id_helper_1.default.getCellAddressString(c, r)),
                        children: this.cellValue(value),
                    },
                },
            },
        };
    }
    slice(tag, length) {
        return {
            children: {
                [tag]: {
                    collection: (collection) => {
                        xml_helper_1.XmlHelper.sliceCollection(collection, length);
                    },
                },
            },
        };
    }
    spanString() {
        return {
            dimension: {
                modify: modify_xml_helper_1.default.attribute('ref', cell_id_helper_1.default.getSpanString(0, 1, this.width, this.height)),
            },
        };
    }
    cellValue(value) {
        return {
            v: {
                modify: modify_xml_helper_1.default.text(String(value)),
            },
        };
    }
    sharedString(label) {
        return this.cellValue(xml_helper_1.XmlHelper.appendSharedString(this.sharedStrings, label));
    }
    setWorkbookTable() {
        this.workbookTable.modify({
            table: {
                modify: modify_xml_helper_1.default.attribute('ref', cell_id_helper_1.default.getSpanString(0, 1, this.width, this.height)),
            },
            tableColumns: {
                modify: modify_xml_helper_1.default.attribute('count', this.width + 1),
            },
        });
        this.columns.forEach((addCol, s) => {
            this.setWorkbookTableColumn(s + 1, addCol.label);
        });
    }
    setWorkbookTableColumn(c, label) {
        this.workbookTable.modify({
            tableColumn: {
                index: c,
                modify: [
                    modify_xml_helper_1.default.attribute('id', c + 1),
                    modify_xml_helper_1.default.attribute('name', label),
                ],
            },
        });
    }
    sliceWorkbookTable() {
        this.workbookTable.modify({
            table: this.slice('tableColumn', this.width + 1),
        });
    }
}
exports.ModifyChart = ModifyChart;
//# sourceMappingURL=modify-chart.js.map