"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModifyTable = void 0;
const xml_helper_1 = require("../helper/xml-helper");
const modify_xml_helper_1 = __importDefault(require("../helper/modify-xml-helper"));
class ModifyTable {
    constructor(table, data) {
        this.row = (index, children) => {
            return {
                'a:tr': {
                    index: index,
                    children: children
                }
            };
        };
        this.column = (index, children) => {
            return {
                'a:tc': {
                    index: index,
                    children: children
                }
            };
        };
        this.cell = (value) => {
            return {
                'a:t': {
                    modify: modify_xml_helper_1.default.text(value),
                }
            };
        };
        this.data = data;
        this.table = new modify_xml_helper_1.default(table);
        this.xml = table;
    }
    modify() {
        this.setContents();
        this.sliceRows();
        return this;
    }
    setContents() {
        this.data.body.forEach((row, r) => {
            row.values.forEach((cell, c) => {
                this.table.modify(this.row(r, this.column(c, this.cell(cell))));
            });
        });
    }
    sliceRows() {
        this.table.modify({
            'a:tbl': this.slice('a:tr', this.data.body.length)
        });
    }
    slice(tag, length) {
        return {
            children: {
                [tag]: {
                    collection: (collection) => {
                        xml_helper_1.XmlHelper.sliceCollection(collection, length);
                    },
                },
            },
        };
    }
    adjustHeight() {
        const tableHeight = Number(this.xml
            .getElementsByTagName('p:xfrm')[0]
            .getElementsByTagName('a:ext')[0]
            .getAttribute('cy'));
        const rowHeight = tableHeight / this.data.body.length;
        this.data.body.forEach((row, r) => {
            this.table.modify({
                'a:tr': {
                    index: r,
                    modify: modify_xml_helper_1.default.attribute('h', Math.round(rowHeight)),
                }
            });
        });
    }
}
exports.ModifyTable = ModifyTable;
//# sourceMappingURL=modify-table.js.map